
/* loadui.c */

#include "loadui.h"
#include "style.h"

#include <QtXml/QDomDocument>

#include <kfiledialog.h>
// #include <kxmlguiwindow.h>
#include <kxmlguifactory.h>

#include <kaction.h>
#include <kactioncollection.h>
#include <kdebug.h>
#include <klocale.h>

#include <sublime/mainwindow.h>

#include <shell/core.h>
#include <shell/uicontroller.h>

/* ----------------------------------------------------------------------- */

namespace Style
{

/* ----------------------------------------------------------------------- */

void UiModule::loadUI()
{
   QString filename =
      KFileDialog::getOpenFileName (KUrl (), "text/xml", NULL, i18n ("Load User Interface"));

   if (filename != "")
   {
      QFile file (filename);
      if (!file.open (QIODevice::ReadOnly))
      {
         kError () << "Could not read " << filename << endl;
      }
      else
      {
         QDomDocument doc;
         QString errMsg;
         int errLine;
         if (!doc.setContent (&file, &errMsg, &errLine))
         {
            kError () << "Parse error: " <<  errMsg  << " in line " << errLine << endl;
         }
         else
         {
            QMap < QString, KAction * > action_dict;

            foreach (KXMLGUIClient* client, KDevelop::Core::self()->uiControllerInternal()->mainWindows()[0]->guiFactory()->clients())
            {
               KActionCollection * c = client->actionCollection();
               for (int i = 0; i < c->count(); i ++)
               {
                  KAction * b = qobject_cast<KAction*> (c->action (i));
                  if (b != NULL)
                     action_dict.insert (b->objectName (), b);
               }
            }

            kDebug () << "Read elements" << endl;
            QDomNode n = doc.documentElement().firstChild ();
            while (! n.isNull ())
            {
               if (n.isElement ())
               {
                  QDomElement e = n.toElement ();
                  kDebug () << "Element: " << e.tagName () << " " << e.attribute ("name") << endl;
                  if (e.tagName () == "ActionProperties")
                  {
                     QDomNode nn = e.firstChild ();
                     while (! nn.isNull ())
                     {
                        if (nn.isElement ())
                        {
                           QDomElement ee = nn.toElement ();
                           kDebug () << "Sub element: " << ee.tagName () << " " << ee.attribute ("name") << endl;
                           if (ee.tagName () == "Action")
                           {
                              KAction * b = action_dict.value (ee.attribute ("name"));
                              if (b != NULL)
                              {
                                 QString s = ee.attribute ("shortcut");
                                 if (s != "")
                                    b->setShortcut (s);

                                 s = ee.attribute ("icon");
                                 if (s != "")
                                    b->setIcon (QIcon (s));

                                 s = ee.attribute ("text");
                                 if (s != "")
                                    b->setText (s);
                              }

                           }
                        }
                        nn = nn.nextSibling ();
                     }
                  }
               }
               n = n.nextSibling ();
            }
         }
      }
   }
}

/* ----------------------------------------------------------------------- */

void UiModule::saveUI()
{
   QString filename =
      KFileDialog::getSaveFileName (KUrl (), "*.xml", 0, i18n ("Save User Interface"));

   if (filename != "")
   {
      QFile file (filename);
      if (!file.open (QIODevice::WriteOnly))
      {
         kError () << "Could not write to " << filename << endl;
      }
      else
      {
         QDomDocument doc;
         QDomElement top = doc.appendChild (doc.createElement ("UI")).toElement ();

         QDomElement target = top.appendChild (doc.createElement ("ActionProperties")).toElement ();

         foreach (KXMLGUIClient* client, KDevelop::Core::self()->uiControllerInternal()->mainWindows()[0]->guiFactory()->clients())
         {
            QDomElement e = client->domDocument().documentElement().namedItem ("ActionProperties").toElement();
            if (! e.isNull ())
            {
               QDomNode n = e.firstChild ();
               while (! n.isNull ())
               {
                  if (n.isElement ())
                     target.appendChild (n.cloneNode (true));
                  n = n.nextSibling ();
               }
            }
         }

         #if 0
         target = top.appendChild (doc.createElement ("Dupl")).toElement ();
         foreach (KXMLGUIClient* client, KDevelop::Core::self()->uiControllerInternal()->mainWindows()[0]->guiFactory()->clients())
         {
            QDomElement e = client->domDocument().documentElement();
            target.appendChild (e.cloneNode (true));
         }
         #endif

         #if 1
         // from kdelibs-4.5.4/kdeui/widgets/kmainwindow.cpp
         target = top.appendChild (doc.createElement ("All")).toElement ();
         foreach (KXMLGUIClient * client, KDevelop::Core::self()->uiControllerInternal()->mainWindows()[0]->guiFactory()->clients())
         {
             foreach (QAction * action, client->actionCollection()->actions())
             {
                 KAction * kaction = qobject_cast<KAction*>(action);
                 if (kaction != NULL)
                 {

                     QString actionName = kaction->objectName();
                     QString shortcut = kaction->shortcut(KAction::ActiveShortcut).toString();
                     QDomElement act_elem = doc.createElement("Action");
                     act_elem.setAttribute( "name", actionName );
                     act_elem.setAttribute( "shortcut", shortcut );
                     target.appendChild(act_elem);
                  }
             }
         }
         #endif

         QTextStream ts (&file);
         ts << doc;
         file.close();
      }
   }
}

/* ----------------------------------------------------------------------- */

void UiModule::init()
{
   KActionCollection * actions = m_plugin->actionCollection ();
   KAction * action = NULL;

   action = actions->addAction ("load_ui");
   action->setShortcut (Qt::ALT + Qt::SHIFT + Qt::Key_O);
   action->setText (i18n ("&Load UI"));
   connect (action, SIGNAL (triggered (bool)), SLOT (loadUI()));

   action = actions->addAction ("save_ui");
   action->setShortcut (Qt::ALT + Qt::SHIFT + Qt::Key_S);
   action->setText (i18n ("&Save UI"));
   connect (action, SIGNAL (triggered (bool)), SLOT (saveUI()));
}

/* ----------------------------------------------------------------------- */

UiModule::UiModule (StylePlugin * plugin) :
      m_plugin (plugin)
{
   init ();
}

/* ----------------------------------------------------------------------- */

} // end of namespace

#include "loadui.moc"
